-- This is the "real" table, but we want to proxy it.
local _global = {}
_global.server = {}

-- Here's our proxy
ULib.global = {}

-- Based off "RecursiveReadOnlyTables" by VeLoSo (http://lua-users.org/wiki/RecursiveReadOnlyTables)

-- cache the metatables of all existing read-only tables,
-- so our functions can get to them, but user code can't
local metatable_cache = setmetatable( {}, { __mode = "k" } )

local function makepairs( real_table, create, t )
	local function pairs()
		local key, value, cur_key = nil, nil, nil
		local function nexter() -- both args dummy
			key, value = next( real_table, cur_key )
			cur_key = key
			if type( value ) == "table" and not metatable_cache[ value ] then
				real_table[ key ] = create( value, key, t.__loc, t.__ply )
				value = real_table[ key ]
			end
			return key, value
		end
		return nexter -- values 2 and 3 dummy
	end
	return pairs
end

function ULib.globalpairs( t )
	local mt = metatable_cache[ t ]
	if not mt then
		ULib.error( "bad argument #1 to 'globalpairs' (global table expected, got " .. type(t) .. ")" )
		return
	end
	return mt.__pairs()
end



-- Add server- and client-specific stuff
if CLIENT then
	include( "ULib/client/cl_global.lua" )
	ULib.registerClGlobal( _global, metatable_cache, makepairs )
else
	include( "ULib/server/sv_global.lua" )
	ULib.registerSvGlobal( _global, metatable_cache, makepairs )
end
